package dev.shadowsoffire.apotheosis.advancements;

import java.util.Iterator;
import java.util.List;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import java.util.Set;
import net.minecraft.advancements.critereon.SerializationContext;
import net.minecraft.advancements.critereon.ContextAwarePredicate;
import net.minecraft.advancements.critereon.AbstractCriterionTriggerInstance;
import net.minecraft.advancements.CriterionTriggerInstance;
import dev.shadowsoffire.apotheosis.spawn.modifiers.SpawnerModifier;
import dev.shadowsoffire.apotheosis.spawn.spawner.ApothSpawnerTile;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.advancements.critereon.MinMaxBounds;
import dev.shadowsoffire.apotheosis.spawn.modifiers.SpawnerStats;
import net.minecraft.advancements.critereon.DeserializationContext;
import com.google.gson.JsonObject;
import com.google.common.collect.Maps;
import net.minecraft.server.PlayerAdvancements;
import java.util.Map;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.advancements.CriterionTrigger;

public class ModifierTrigger implements CriterionTrigger<Instance>
{
    private static final ResourceLocation ID;
    private final Map<PlayerAdvancements, Listeners> listeners;
    
    public ModifierTrigger() {
        this.listeners = Maps.newHashMap();
    }
    
    public ResourceLocation m_7295_() {
        return ModifierTrigger.ID;
    }
    
    public void m_6467_(final PlayerAdvancements playerAdvancementsIn, final CriterionTrigger.Listener<Instance> listener) {
        Listeners ModifierTrigger$listeners = this.listeners.get(playerAdvancementsIn);
        if (ModifierTrigger$listeners == null) {
            ModifierTrigger$listeners = new Listeners(playerAdvancementsIn);
            this.listeners.put(playerAdvancementsIn, ModifierTrigger$listeners);
        }
        ModifierTrigger$listeners.add(listener);
    }
    
    public void m_6468_(final PlayerAdvancements playerAdvancementsIn, final CriterionTrigger.Listener<Instance> listener) {
        final Listeners ModifierTrigger$listeners = this.listeners.get(playerAdvancementsIn);
        if (ModifierTrigger$listeners != null) {
            ModifierTrigger$listeners.remove(listener);
            if (ModifierTrigger$listeners.isEmpty()) {
                this.listeners.remove(playerAdvancementsIn);
            }
        }
    }
    
    public void m_5656_(final PlayerAdvancements playerAdvancementsIn) {
        this.listeners.remove(playerAdvancementsIn);
    }
    
    public Instance createInstance(final JsonObject json, final DeserializationContext conditionsParser) {
        final MinMaxBounds.Ints minDelay = MinMaxBounds.Ints.m_55373_(json.get(SpawnerStats.MIN_DELAY.getId()));
        final MinMaxBounds.Ints maxDelay = MinMaxBounds.Ints.m_55373_(json.get(SpawnerStats.MAX_DELAY.getId()));
        final MinMaxBounds.Ints spawnCount = MinMaxBounds.Ints.m_55373_(json.get(SpawnerStats.SPAWN_COUNT.getId()));
        final MinMaxBounds.Ints nearbyEnts = MinMaxBounds.Ints.m_55373_(json.get(SpawnerStats.MAX_NEARBY_ENTITIES.getId()));
        final MinMaxBounds.Ints playerRange = MinMaxBounds.Ints.m_55373_(json.get(SpawnerStats.REQ_PLAYER_RANGE.getId()));
        final MinMaxBounds.Ints spawnRange = MinMaxBounds.Ints.m_55373_(json.get(SpawnerStats.SPAWN_RANGE.getId()));
        final Boolean ignorePlayers = json.has(SpawnerStats.IGNORE_PLAYERS.getId()) ? Boolean.valueOf(json.get(SpawnerStats.IGNORE_PLAYERS.getId()).getAsBoolean()) : null;
        final Boolean ignoreConditions = json.has(SpawnerStats.IGNORE_CONDITIONS.getId()) ? Boolean.valueOf(json.get(SpawnerStats.IGNORE_CONDITIONS.getId()).getAsBoolean()) : null;
        final Boolean redstone = json.has(SpawnerStats.REDSTONE_CONTROL.getId()) ? Boolean.valueOf(json.get(SpawnerStats.REDSTONE_CONTROL.getId()).getAsBoolean()) : null;
        final Boolean ignoreLight = json.has(SpawnerStats.IGNORE_LIGHT.getId()) ? Boolean.valueOf(json.get(SpawnerStats.IGNORE_LIGHT.getId()).getAsBoolean()) : null;
        final Boolean noAI = json.has(SpawnerStats.NO_AI.getId()) ? Boolean.valueOf(json.get(SpawnerStats.NO_AI.getId()).getAsBoolean()) : null;
        return new Instance(minDelay, maxDelay, spawnCount, nearbyEnts, playerRange, spawnRange, ignorePlayers, ignoreConditions, redstone, ignoreLight, noAI);
    }
    
    public void trigger(final ServerPlayer player, final ApothSpawnerTile tile, final SpawnerModifier modif) {
        final Listeners ModifierTrigger$listeners = this.listeners.get(player.m_8960_());
        if (ModifierTrigger$listeners != null) {
            ModifierTrigger$listeners.trigger(tile, modif);
        }
    }
    
    static {
        ID = new ResourceLocation("apotheosis", "spawner_modifier");
    }
    
    public static class Instance extends AbstractCriterionTriggerInstance
    {
        private final MinMaxBounds.Ints minDelay;
        private final MinMaxBounds.Ints maxDelay;
        private final MinMaxBounds.Ints spawnCount;
        private final MinMaxBounds.Ints nearbyEnts;
        private final MinMaxBounds.Ints playerRange;
        private final MinMaxBounds.Ints spawnRange;
        private final Boolean ignorePlayers;
        private final Boolean ignoreConditions;
        private final Boolean redstone;
        private final Boolean ignoreLight;
        private final Boolean noAI;
        
        public Instance(final MinMaxBounds.Ints minDelay, final MinMaxBounds.Ints maxDelay, final MinMaxBounds.Ints spawnCount, final MinMaxBounds.Ints nearbyEnts, final MinMaxBounds.Ints playerRange, final MinMaxBounds.Ints spawnRange, final Boolean ignorePlayers, final Boolean ignoreConditions, final Boolean redstone, final Boolean ignoreLight, final Boolean noAI) {
            super(ModifierTrigger.ID, ContextAwarePredicate.f_285567_);
            this.minDelay = minDelay;
            this.maxDelay = maxDelay;
            this.spawnCount = spawnCount;
            this.nearbyEnts = nearbyEnts;
            this.playerRange = playerRange;
            this.spawnRange = spawnRange;
            this.ignorePlayers = ignorePlayers;
            this.ignoreConditions = ignoreConditions;
            this.redstone = redstone;
            this.ignoreLight = ignoreLight;
            this.noAI = noAI;
        }
        
        public JsonObject m_7683_(final SerializationContext serializer) {
            return new JsonObject();
        }
        
        public boolean test(final ApothSpawnerTile tile, final SpawnerModifier modif) {
            final ApothSpawnerTile.SpawnerLogicExt logic = (ApothSpawnerTile.SpawnerLogicExt)tile.f_59788_;
            return this.minDelay.m_55390_(logic.f_45447_) && this.maxDelay.m_55390_(logic.f_45448_) && this.spawnCount.m_55390_(logic.f_45449_) && this.nearbyEnts.m_55390_(logic.f_45451_) && this.playerRange.m_55390_(logic.f_45452_) && this.spawnRange.m_55390_(logic.f_45453_) && (this.ignorePlayers == null || tile.ignoresPlayers == this.ignorePlayers) && (this.ignoreConditions == null || tile.ignoresConditions == this.ignoreConditions) && (this.redstone == null || tile.redstoneControl == this.redstone) && (this.ignoreLight == null || tile.ignoresLight == this.ignoreLight) && (this.noAI == null || tile.hasNoAI == this.noAI);
        }
    }
    
    static class Listeners
    {
        private final PlayerAdvancements playerAdvancements;
        private final Set<CriterionTrigger.Listener<Instance>> listeners;
        
        public Listeners(final PlayerAdvancements playerAdvancementsIn) {
            this.listeners = Sets.newHashSet();
            this.playerAdvancements = playerAdvancementsIn;
        }
        
        public boolean isEmpty() {
            return this.listeners.isEmpty();
        }
        
        public void add(final CriterionTrigger.Listener<Instance> listener) {
            this.listeners.add(listener);
        }
        
        public void remove(final CriterionTrigger.Listener<Instance> listener) {
            this.listeners.remove(listener);
        }
        
        public void trigger(final ApothSpawnerTile tile, final SpawnerModifier modif) {
            List<CriterionTrigger.Listener<Instance>> list = null;
            for (final CriterionTrigger.Listener<Instance> listener : this.listeners) {
                if (((Instance)listener.m_13685_()).test(tile, modif)) {
                    if (list == null) {
                        list = Lists.newArrayList();
                    }
                    list.add(listener);
                }
            }
            if (list != null) {
                for (final CriterionTrigger.Listener<Instance> listener2 : list) {
                    listener2.m_13686_(this.playerAdvancements);
                }
            }
        }
    }
}
